@@ -183,7 +183,7 @@ use log::{debug, error, warn};
 extern crate lazy_static;
 
 pub const SERVER_ID: &str = "yedb-altt-rs";
-pub const VERSION: &str = "0.0.15";
+pub const VERSION: &str = "0.0.16";
 pub const ENGINE_VERSION: u8 = 1;
 
 pub const DEFAULT_CACHE_SIZE: usize = 1000;
@@ -587,6 +587,7 @@ pub struct Database {
     pub timeout: Duration,
     pub lock_ex: bool,
     pub auto_bak: u64,
+    pub strict_schema: bool,
     default_fmt: SerializationEngine,
     default_checksums: bool,
     meta_path: String,
@@ -615,6 +616,7 @@ impl Database {
             auto_repair: true,
             auto_flush: true,
             auto_bak: 0,
+            strict_schema: false,
             lock_ex: true,
             write_modified_only: true,
             timeout: Duration::from_secs(5),
@@ -846,7 +848,14 @@ impl Database {
                         .compile(&schema));
                     unwrap_schema_validate!(compiled.validate(value));
                 }
-                None => {}
+                None => {
+                    if self.strict_schema {
+                        return Err(Error::new(
+                            ErrorKind::SchemaValidationError,
+                            "schema not defined",
+                        ));
+                    }
+                }
             }
         }
         Ok(())
@@ -1328,6 +1337,7 @@ impl Database {
             cached_keys: self.cache.len(),
             cache_size: self.cache.cap(),
             auto_bak: self.auto_bak,
+            strict_schema: self.strict_schema,
             path: self.path.clone(),
             lock_path: self.lock_path.clone(),
             server: (SERVER_ID.to_owned(), VERSION.to_owned()),
@@ -1357,6 +1367,10 @@ impl Database {
                 Some(v) => self.auto_bak = v,
                 _ => invalid_server_option_value!(name, &value),
             },
+            "strict_schema" => match value {
+                Value::Bool(v) => self.strict_schema = v,
+                _ => invalid_server_option_value!(name, &value),
+            },
             "repair_recommended" => match value {
                 Value::Bool(v) => self.repair_recommended = v,
                 _ => invalid_server_option_value!(name, &value),
@@ -1398,7 +1412,7 @@ impl Database {
     }
 
     pub fn key_delete(&mut self, key: &str) -> Result<(), Error> {
-        if self.auto_bak > 0 {
+        if self.auto_bak > 0 && !key.starts_with(".schema/") && key != ".schema" {
             for n in 1..self.auto_bak + 1 {
                 let key_name = format!("{}.bak{}", key, n);
                 match self._delete_key(&key_name, false, false, false) {
@@ -1589,7 +1603,8 @@ impl Database {
     }
 
     pub fn key_set(&mut self, key: &str, value: Value) -> Result<(), Error> {
-        if self.auto_bak > 0 {
+        self.validate_schema(&key, &value)?;
+        if self.auto_bak > 0 && !key.starts_with(".schema/") && key != "schema" {
             for n in (1..self.auto_bak + 1).rev() {
                 let key_from = match n {
                     1 => key.to_owned(),
@@ -1603,7 +1618,7 @@ impl Database {
                 }
             }
         }
-        return self.set_key_data(key, value, None, false);
+        return self.set_key_data(key, value, None, true);
     }
 
     pub fn key_list(&mut self, key: &str) -> Result<Vec<String>, Error> {
@@ -1615,7 +1630,7 @@ impl Database {
     pub fn key_copy(&mut self, key: &str, dst_key: &str) -> Result<(), Error> {
         debug!("Copying key {} to {}", key, dst_key);
         let value = self.get_key_data(DataKey::Name(key), false)?.0;
-        return self.set_key_data(dst_key, value, None, true);
+        return self.set_key_data(dst_key, value, None, false);
     }
 
     pub fn key_increment(&mut self, key: &str) -> Result<i64, Error> {
